@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "loopbackdevicelink.h"
@@ -35,7 +35,7 @@ QString LoopbackDeviceLink::name()
 
 bool LoopbackDeviceLink::sendPacket(NetworkPacket& input)
 {
-    NetworkPacket output(QString{});
+    NetworkPacket output((QString()));
     NetworkPacket::unserialize(input.serialize(), &output);
 
     //LoopbackDeviceLink does not need deviceTransferInfo
