@@ -15,16 +15,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "device.h"
 
-#ifdef interface // MSVC language extension, QDBusConnection uses this as a variable name
-#undef interface
-#endif
-
-#include <QDBusConnection>
+#include <QVector>
+#include <QSet>
 #include <QSslCertificate>
 
 #include <KSharedConfig>
@@ -37,9 +34,42 @@
 #include "backends/devicelink.h"
 #include "backends/lan/landevicelink.h"
 #include "backends/linkprovider.h"
-#include "networkpackage.h"
+#include "networkpacket.h"
 #include "kdeconnectconfig.h"
 #include "daemon.h"
+#include "dbushelper.h"
+
+//In older Qt released, qAsConst isnt available
+#include "qtcompat_p.h"
+
+class Device::DevicePrivate
+{
+public:
+    DevicePrivate(const QString &id)
+        : m_deviceId(id)
+    {
+
+    }
+
+    ~DevicePrivate()
+    {
+        qDeleteAll(m_deviceLinks);
+        m_deviceLinks.clear();
+    }
+
+    const QString m_deviceId;
+    QString m_deviceName;
+    DeviceType m_deviceType;
+    int m_protocolVersion;
+
+    QVector<DeviceLink *> m_deviceLinks;
+    QHash<QString, KdeConnectPlugin *> m_plugins;
+
+    QMultiMap<QString, KdeConnectPlugin *> m_pluginsByIncomingCapability;
+    QSet<QString> m_supportedPlugins;
+    QSet<QString> m_allPlugins;
+    QSet<PairingHandler *> m_pairRequests;
+};
 
 static void warn(const QString& info)
 {
@@ -48,69 +78,104 @@ static void warn(const QString& info)
 
 Device::Device(QObject* parent, const QString& id)
     : QObject(parent)
-    , m_deviceId(id)
-    , m_protocolVersion(NetworkPackage::s_protocolVersion) //We don't know it yet
+    , d(new Device::DevicePrivate(id))
 {
-    KdeConnectConfig::DeviceInfo info = KdeConnectConfig::instance()->getTrustedDevice(id);
+    d->m_protocolVersion = NetworkPacket::s_protocolVersion;
+    KdeConnectConfig::DeviceInfo info = KdeConnectConfig::instance().getTrustedDevice(d->m_deviceId);
 
-    m_deviceName = info.deviceName;
-    m_deviceType = str2type(info.deviceType);
+    d->m_deviceName = info.deviceName;
+    d->m_deviceType = str2type(info.deviceType);
 
     //Register in bus
-    QDBusConnection::sessionBus().registerObject(dbusPath(), this, QDBusConnection::ExportScriptableContents | QDBusConnection::ExportAdaptors);
+    DBusHelper::sessionBus().registerObject(dbusPath(), this, QDBusConnection::ExportScriptableContents | QDBusConnection::ExportAdaptors);
 
     //Assume every plugin is supported until addLink is called and we can get the actual list
-    m_supportedPlugins = PluginLoader::instance()->getPluginList().toSet();
+    d->m_allPlugins = PluginLoader::instance()->getPluginList().toSet();
+    d->m_supportedPlugins = d->m_allPlugins;
 
     connect(this, &Device::pairingError, this, &warn);
 }
 
-Device::Device(QObject* parent, const NetworkPackage& identityPackage, DeviceLink* dl)
+Device::Device(QObject* parent, const NetworkPacket& identityPacket, DeviceLink* dl)
     : QObject(parent)
-    , m_deviceId(identityPackage.get<QString>(QStringLiteral("deviceId")))
-    , m_deviceName(identityPackage.get<QString>(QStringLiteral("deviceName")))
+    , d(new Device::DevicePrivate(identityPacket.get<QString>(QStringLiteral("deviceId"))))
 {
-    addLink(identityPackage, dl);
+    d->m_deviceName = identityPacket.get<QString>(QStringLiteral("deviceName"));
+    d->m_allPlugins = PluginLoader::instance()->getPluginList().toSet();
+
+    addLink(identityPacket, dl);
 
     //Register in bus
-    QDBusConnection::sessionBus().registerObject(dbusPath(), this, QDBusConnection::ExportScriptableContents | QDBusConnection::ExportAdaptors);
+    DBusHelper::sessionBus().registerObject(dbusPath(), this, QDBusConnection::ExportScriptableContents | QDBusConnection::ExportAdaptors);
 
     connect(this, &Device::pairingError, this, &warn);
+
+    connect(this, &Device::reachableChanged, this, &Device::statusIconNameChanged);
+    connect(this, &Device::trustedChanged, this, &Device::statusIconNameChanged);
 }
 
 Device::~Device()
 {
-    qDeleteAll(m_deviceLinks);
-    m_deviceLinks.clear();
+    delete d;
+}
+
+QString Device::id() const
+{
+    return d->m_deviceId;
+}
+
+QString Device::name() const
+{
+    return d->m_deviceName;
+}
+
+QString Device::type() const
+{
+    return type2str(d->m_deviceType);
+}
+
+bool Device::isReachable() const
+{
+    return !d->m_deviceLinks.isEmpty();
+}
+
+int Device::protocolVersion()
+{
+    return d->m_protocolVersion;
+}
+
+QStringList Device::supportedPlugins() const
+{
+    return d->m_supportedPlugins.toList();
 }
 
 bool Device::hasPlugin(const QString& name) const
 {
-    return m_plugins.contains(name);
+    return d->m_plugins.contains(name);
 }
 
 QStringList Device::loadedPlugins() const
 {
-    return m_plugins.keys();
+    return d->m_plugins.keys();
 }
 
 void Device::reloadPlugins()
 {
-    QHash<QString, KdeConnectPlugin*> newPluginMap, oldPluginMap = m_plugins;
+    QHash<QString, KdeConnectPlugin*> newPluginMap, oldPluginMap = d->m_plugins;
     QMultiMap<QString, KdeConnectPlugin*> newPluginsByIncomingCapability;
 
     if (isTrusted() && isReachable()) { //Do not load any plugin for unpaired devices, nor useless loading them for unreachable devices
 
         PluginLoader* loader = PluginLoader::instance();
 
-        for (const QString& pluginName : qAsConst(m_supportedPlugins)) {
+        for (const QString& pluginName : qAsConst(d->m_supportedPlugins)) {
             const KPluginMetaData service = loader->getPluginInfo(pluginName);
 
             const bool pluginEnabled = isPluginEnabled(pluginName);
-            const QSet<QString> incomingCapabilities = KPluginMetaData::readStringList(service.rawData(), QStringLiteral("X-KdeConnect-SupportedPackageType")).toSet();
+            const QSet<QString> incomingCapabilities = KPluginMetaData::readStringList(service.rawData(), QStringLiteral("X-KdeConnect-SupportedPacketType")).toSet();
 
             if (pluginEnabled) {
-                KdeConnectPlugin* plugin = m_plugins.take(pluginName);
+                KdeConnectPlugin* plugin = d->m_plugins.take(pluginName);
 
                 if (!plugin) {
                     plugin = loader->instantiatePluginForDevice(pluginName, this);
@@ -130,12 +195,12 @@ void Device::reloadPlugins()
 
     //Erase all left plugins in the original map (meaning that we don't want
     //them anymore, otherwise they would have been moved to the newPluginMap)
-    qDeleteAll(m_plugins);
-    m_plugins = newPluginMap;
-    m_pluginsByIncomingCapability = newPluginsByIncomingCapability;
+    qDeleteAll(d->m_plugins);
+    d->m_plugins = newPluginMap;
+    d->m_pluginsByIncomingCapability = newPluginsByIncomingCapability;
 
-    QDBusConnection bus = QDBusConnection::sessionBus();
-    for (KdeConnectPlugin* plugin : qAsConst(m_plugins)) {
+    QDBusConnection bus = DBusHelper::sessionBus();
+    for (KdeConnectPlugin* plugin : qAsConst(d->m_plugins)) {
         //TODO: see how it works in Android (only done once, when created)
         plugin->connected();
 
@@ -151,7 +216,7 @@ void Device::reloadPlugins()
 
 QString Device::pluginsConfigFile() const
 {
-    return KdeConnectConfig::instance()->deviceConfigDir(id()).absoluteFilePath(QStringLiteral("config"));
+    return KdeConnectConfig::instance().deviceConfigDir(id()).absoluteFilePath(QStringLiteral("config"));
 }
 
 void Device::requestPair()
@@ -166,32 +231,32 @@ void Device::requestPair()
         return;
     }
 
-    for (DeviceLink* dl : qAsConst(m_deviceLinks)) {
+    for (DeviceLink* dl : qAsConst(d->m_deviceLinks)) {
         dl->userRequestsPair();
     }
 }
 
 void Device::unpair()
 {
-    for (DeviceLink* dl : qAsConst(m_deviceLinks)) {
+    for (DeviceLink* dl : qAsConst(d->m_deviceLinks)) {
         dl->userRequestsUnpair();
     }
-    KdeConnectConfig::instance()->removeTrustedDevice(id());
+    KdeConnectConfig::instance().removeTrustedDevice(id());
     Q_EMIT trustedChanged(false);
 }
 
 void Device::pairStatusChanged(DeviceLink::PairStatus status)
 {
     if (status == DeviceLink::NotPaired) {
-        KdeConnectConfig::instance()->removeTrustedDevice(id());
+        KdeConnectConfig::instance().removeTrustedDevice(id());
 
-        for (DeviceLink* dl : qAsConst(m_deviceLinks)) {
+        for (DeviceLink* dl : qAsConst(d->m_deviceLinks)) {
             if (dl != sender()) {
                 dl->setPairStatus(DeviceLink::NotPaired);
             }
         }
     } else {
-        KdeConnectConfig::instance()->addTrustedDevice(id(), name(), type());
+        KdeConnectConfig::instance().addTrustedDevice(id(), name(), type());
     }
 
     reloadPlugins(); //Will load/unload plugins
@@ -206,49 +271,50 @@ static bool lessThan(DeviceLink* p1, DeviceLink* p2)
     return p1->provider()->priority() > p2->provider()->priority();
 }
 
-void Device::addLink(const NetworkPackage& identityPackage, DeviceLink* link)
+void Device::addLink(const NetworkPacket& identityPacket, DeviceLink* link)
 {
     //qCDebug(KDECONNECT_CORE) << "Adding link to" << id() << "via" << link->provider();
 
-    setName(identityPackage.get<QString>(QStringLiteral("deviceName")));
-    m_deviceType = str2type(identityPackage.get<QString>(QStringLiteral("deviceType")));
+    setName(identityPacket.get<QString>(QStringLiteral("deviceName")));
+    d->m_deviceType = str2type(identityPacket.get<QString>(QStringLiteral("deviceType")));
 
-    if (m_deviceLinks.contains(link))
+    if (d->m_deviceLinks.contains(link)) {
         return;
+    }
 
-    m_protocolVersion = identityPackage.get<int>(QStringLiteral("protocolVersion"), -1);
-    if (m_protocolVersion != NetworkPackage::s_protocolVersion) {
-        qCWarning(KDECONNECT_CORE) << m_deviceName << "- warning, device uses a different protocol version" << m_protocolVersion << "expected" << NetworkPackage::s_protocolVersion;
+    d->m_protocolVersion = identityPacket.get<int>(QStringLiteral("protocolVersion"), -1);
+    if (d->m_protocolVersion != NetworkPacket::s_protocolVersion) {
+        qCWarning(KDECONNECT_CORE) << d->m_deviceName << "- warning, device uses a different protocol version" << d->m_protocolVersion << "expected" << NetworkPacket::s_protocolVersion;
     }
 
     connect(link, &QObject::destroyed,
             this, &Device::linkDestroyed);
 
-    m_deviceLinks.append(link);
+    d->m_deviceLinks.append(link);
 
     //Theoretically we will never add two links from the same provider (the provider should destroy
     //the old one before this is called), so we do not have to worry about destroying old links.
     //-- Actually, we should not destroy them or the provider will store an invalid ref!
 
-    connect(link, &DeviceLink::receivedPackage,
-            this, &Device::privateReceivedPackage);
+    connect(link, &DeviceLink::receivedPacket,
+            this, &Device::privateReceivedPacket);
 
-    qSort(m_deviceLinks.begin(), m_deviceLinks.end(), lessThan);
+    std::sort(d->m_deviceLinks.begin(), d->m_deviceLinks.end(), lessThan);
 
-    const bool capabilitiesSupported = identityPackage.has(QStringLiteral("incomingCapabilities")) || identityPackage.has(QStringLiteral("outgoingCapabilities"));
+    const bool capabilitiesSupported = identityPacket.has(QStringLiteral("incomingCapabilities")) || identityPacket.has(QStringLiteral("outgoingCapabilities"));
     if (capabilitiesSupported) {
-        const QSet<QString> outgoingCapabilities = identityPackage.get<QStringList>(QStringLiteral("outgoingCapabilities")).toSet()
-                          , incomingCapabilities = identityPackage.get<QStringList>(QStringLiteral("incomingCapabilities")).toSet();
+        const QSet<QString> outgoingCapabilities = identityPacket.get<QStringList>(QStringLiteral("outgoingCapabilities")).toSet()
+                          , incomingCapabilities = identityPacket.get<QStringList>(QStringLiteral("incomingCapabilities")).toSet();
 
-        m_supportedPlugins = PluginLoader::instance()->pluginsForCapabilities(incomingCapabilities, outgoingCapabilities);
+        d->m_supportedPlugins = PluginLoader::instance()->pluginsForCapabilities(incomingCapabilities, outgoingCapabilities);
         //qDebug() << "new plugins for" << m_deviceName << m_supportedPlugins << incomingCapabilities << outgoingCapabilities;
     } else {
-        m_supportedPlugins = PluginLoader::instance()->getPluginList().toSet();
+        d->m_supportedPlugins = PluginLoader::instance()->getPluginList().toSet();
     }
 
     reloadPlugins();
 
-    if (m_deviceLinks.size() == 1) {
+    if (d->m_deviceLinks.size() == 1) {
         Q_EMIT reachableChanged(true);
     }
 
@@ -260,45 +326,45 @@ void Device::addLink(const NetworkPackage& identityPackage, DeviceLink* link)
 
 void Device::addPairingRequest(PairingHandler* handler)
 {
-    const bool wasEmpty = m_pairRequests.isEmpty();
-    m_pairRequests.insert(handler);
+    const bool wasEmpty = d->m_pairRequests.isEmpty();
+    d->m_pairRequests.insert(handler);
 
-    if (wasEmpty != m_pairRequests.isEmpty())
-        Q_EMIT hasPairingRequestsChanged(!m_pairRequests.isEmpty());
+    if (wasEmpty != d->m_pairRequests.isEmpty())
+        Q_EMIT hasPairingRequestsChanged(!d->m_pairRequests.isEmpty());
 }
 
 void Device::removePairingRequest(PairingHandler* handler)
 {
-    const bool wasEmpty = m_pairRequests.isEmpty();
-    m_pairRequests.remove(handler);
+    const bool wasEmpty = d->m_pairRequests.isEmpty();
+    d->m_pairRequests.remove(handler);
 
-    if (wasEmpty != m_pairRequests.isEmpty())
-        Q_EMIT hasPairingRequestsChanged(!m_pairRequests.isEmpty());
+    if (wasEmpty != d->m_pairRequests.isEmpty())
+        Q_EMIT hasPairingRequestsChanged(!d->m_pairRequests.isEmpty());
 }
 
 bool Device::hasPairingRequests() const
 {
-    return !m_pairRequests.isEmpty();
+    return !d->m_pairRequests.isEmpty();
 }
 
 void Device::acceptPairing()
 {
-    if (m_pairRequests.isEmpty())
+    if (d->m_pairRequests.isEmpty())
         qWarning() << "no pair requests to accept!";
 
     //copying because the pairing handler will be removed upon accept
-    const auto prCopy = m_pairRequests;
+    const auto prCopy = d->m_pairRequests;
     for (auto ph: prCopy)
         ph->acceptPairing();
 }
 
 void Device::rejectPairing()
 {
-    if (m_pairRequests.isEmpty())
+    if (d->m_pairRequests.isEmpty())
         qWarning() << "no pair requests to reject!";
 
     //copying because the pairing handler will be removed upon reject
-    const auto prCopy = m_pairRequests;
+    const auto prCopy = d->m_pairRequests;
     for (auto ph: prCopy)
         ph->rejectPairing();
 }
@@ -310,42 +376,42 @@ void Device::linkDestroyed(QObject* o)
 
 void Device::removeLink(DeviceLink* link)
 {
-    m_deviceLinks.removeAll(link);
+    d->m_deviceLinks.removeAll(link);
 
     //qCDebug(KDECONNECT_CORE) << "RemoveLink" << m_deviceLinks.size() << "links remaining";
 
-    if (m_deviceLinks.isEmpty()) {
+    if (d->m_deviceLinks.isEmpty()) {
         reloadPlugins();
         Q_EMIT reachableChanged(false);
     }
 }
 
-bool Device::sendPackage(NetworkPackage& np)
+bool Device::sendPacket(NetworkPacket& np)
 {
-    Q_ASSERT(np.type() != PACKAGE_TYPE_PAIR);
+    Q_ASSERT(np.type() != PACKET_TYPE_PAIR);
     Q_ASSERT(isTrusted());
 
-    //Maybe we could block here any package that is not an identity or a pairing package to prevent sending non encrypted data
-    for (DeviceLink* dl : qAsConst(m_deviceLinks)) {
-        if (dl->sendPackage(np)) return true;
+    //Maybe we could block here any packet that is not an identity or a pairing packet to prevent sending non encrypted data
+    for (DeviceLink* dl : qAsConst(d->m_deviceLinks)) {
+        if (dl->sendPacket(np)) return true;
     }
 
     return false;
 }
 
-void Device::privateReceivedPackage(const NetworkPackage& np)
+void Device::privateReceivedPacket(const NetworkPacket& np)
 {
-    Q_ASSERT(np.type() != PACKAGE_TYPE_PAIR);
+    Q_ASSERT(np.type() != PACKET_TYPE_PAIR);
     if (isTrusted()) {
-        const QList<KdeConnectPlugin*> plugins = m_pluginsByIncomingCapability.values(np.type());
+        const QList<KdeConnectPlugin*> plugins = d->m_pluginsByIncomingCapability.values(np.type());
         if (plugins.isEmpty()) {
-            qWarning() << "discarding unsupported package" << np.type() << "for" << name();
+            qWarning() << "discarding unsupported packet" << np.type() << "for" << name();
         }
         for (KdeConnectPlugin* plugin : plugins) {
-            plugin->receivePackage(np);
+            plugin->receivePacket(np);
         }
     } else {
-        qCDebug(KDECONNECT_CORE) << "device" << name() << "not paired, ignoring package" << np.type();
+        qCDebug(KDECONNECT_CORE) << "device" << name() << "not paired, ignoring packet" << np.type();
         unpair();
     }
 
@@ -353,14 +419,14 @@ void Device::privateReceivedPackage(const NetworkPackage& np)
 
 bool Device::isTrusted() const
 {
-    return KdeConnectConfig::instance()->trustedDevices().contains(id());
+    return KdeConnectConfig::instance().trustedDevices().contains(id());
 }
 
 QStringList Device::availableLinks() const
 {
     QStringList sl;
-    sl.reserve(m_deviceLinks.size());
-    for (DeviceLink* dl : qAsConst(m_deviceLinks)) {
+    sl.reserve(d->m_deviceLinks.size());
+    for (DeviceLink* dl : qAsConst(d->m_deviceLinks)) {
         sl.append(dl->provider()->name());
     }
     return sl;
@@ -370,11 +436,11 @@ void Device::cleanUnneededLinks() {
     if (isTrusted()) {
         return;
     }
-    for(int i = 0; i < m_deviceLinks.size(); ) {
-        DeviceLink* dl = m_deviceLinks[i];
+    for(int i = 0; i < d->m_deviceLinks.size(); ) {
+        DeviceLink* dl = d->m_deviceLinks[i];
         if (!dl->linkShouldBeKeptAlive()) {
             dl->deleteLater();
-            m_deviceLinks.remove(i);
+            d->m_deviceLinks.remove(i);
         } else {
             i++;
         }
@@ -383,7 +449,7 @@ void Device::cleanUnneededLinks() {
 
 QHostAddress Device::getLocalIpAddress() const
 {
-    for (DeviceLink* dl : m_deviceLinks) {
+    for (DeviceLink* dl : qAsConst(d->m_deviceLinks)) {
         LanDeviceLink* ldl = dynamic_cast<LanDeviceLink*>(dl);
         if (ldl) {
             return ldl->hostAddress();
@@ -397,6 +463,7 @@ Device::DeviceType Device::str2type(const QString& deviceType) {
     if (deviceType == QLatin1String("laptop")) return Laptop;
     if (deviceType == QLatin1String("smartphone") || deviceType == QLatin1String("phone")) return Phone;
     if (deviceType == QLatin1String("tablet")) return Tablet;
+    if (deviceType == QLatin1String("tv")) return Tv;
     return Unknown;
 }
 
@@ -405,6 +472,7 @@ QString Device::type2str(Device::DeviceType deviceType) {
     if (deviceType == Laptop) return QStringLiteral("laptop");
     if (deviceType == Phone) return QStringLiteral("smartphone");
     if (deviceType == Tablet) return QStringLiteral("tablet");
+    if (deviceType == Tv) return QStringLiteral("tv");
     return QStringLiteral("unknown");
 }
 
@@ -421,7 +489,7 @@ QString Device::iconName() const
 
 QString Device::iconForStatus(bool reachable, bool trusted) const
 {
-    Device::DeviceType deviceType = m_deviceType;
+    Device::DeviceType deviceType = d->m_deviceType;
     if (deviceType == Device::Unknown) {
         deviceType = Device::Phone; //Assume phone if we don't know the type
     } else if (deviceType == Device::Desktop) {
@@ -436,19 +504,23 @@ QString Device::iconForStatus(bool reachable, bool trusted) const
 
 void Device::setName(const QString& name)
 {
-    if (m_deviceName != name) {
-        m_deviceName = name;
+    if (d->m_deviceName != name) {
+        d->m_deviceName = name;
         Q_EMIT nameChanged(name);
     }
 }
 
 KdeConnectPlugin* Device::plugin(const QString& pluginName) const
 {
-    return m_plugins[pluginName];
+    return d->m_plugins[pluginName];
 }
 
 void Device::setPluginEnabled(const QString& pluginName, bool enabled)
 {
+    if (!d->m_allPlugins.contains(pluginName)) {
+        return;
+    }
+
     KConfigGroup pluginStates = KSharedConfig::openConfig(pluginsConfigFile())->group("Plugins");
 
     const QString enabledKey = pluginName + QStringLiteral("Enabled");
@@ -470,20 +542,27 @@ QString Device::encryptionInfo() const
     QString result;
     QCryptographicHash::Algorithm digestAlgorithm = QCryptographicHash::Algorithm::Sha1;
 
-    QString localSha1 = QString::fromLatin1(KdeConnectConfig::instance()->certificate().digest(digestAlgorithm).toHex());
+    QString localSha1 = QString::fromLatin1(KdeConnectConfig::instance().certificate().digest(digestAlgorithm).toHex());
     for (int i = 2; i<localSha1.size(); i += 3) {
-        localSha1.insert(i, ':'); // Improve readability
+        localSha1.insert(i, QStringLiteral(":")); // Improve readability
     }
     result += i18n("SHA1 fingerprint of your device certificate is: %1\n", localSha1);
 
-    std::string  remotePem = KdeConnectConfig::instance()->getDeviceProperty(id(), QStringLiteral("certificate")).toStdString();
+    std::string  remotePem = KdeConnectConfig::instance().getDeviceProperty(id(), QStringLiteral("certificate")).toStdString();
     QSslCertificate remoteCertificate = QSslCertificate(QByteArray(remotePem.c_str(), (int)remotePem.size()));
     QString remoteSha1 = QString::fromLatin1(remoteCertificate.digest(digestAlgorithm).toHex());
     for (int i = 2; i < remoteSha1.size(); i += 3) {
-        remoteSha1.insert(i, ':'); // Improve readability
+        remoteSha1.insert(i, QStringLiteral(":")); // Improve readability
     }
     result += i18n("SHA1 fingerprint of remote device certificate is: %1\n", remoteSha1);
 
     return result;
 }
 
+QString Device::pluginIconName(const QString& pluginName)
+{
+    if (hasPlugin(pluginName)) {
+        return d->m_plugins[pluginName]->iconName();
+    }
+    return QString();
+}
