@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "kdeconnectconfig.h"
@@ -29,16 +29,17 @@
 #include <QDir>
 #include <QStandardPaths>
 #include <QCoreApplication>
-#include <QHostInfo>
 #include <QSettings>
 #include <QSslCertificate>
 #include <QtCrypto>
-#include <QSslCertificate>
+#include <QThread>
 
 #include "core_debug.h"
 #include "dbushelper.h"
 #include "daemon.h"
 
+const QFile::Permissions strictPermissions = QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::WriteUser;
+
 struct KdeConnectConfigPrivate {
 
     // The Initializer object sets things up, and also does cleanup when it goes out of scope
@@ -51,11 +52,17 @@ struct KdeConnectConfigPrivate {
     QSettings* m_config;
     QSettings* m_trustedDevices;
 
+    QString m_defaultName;
+    QString m_deviceType;
+
+#ifdef USE_PRIVATE_DBUS
+    QString m_privateDBusAddress;  // Private DBus Address cache
+#endif
 };
 
-KdeConnectConfig* KdeConnectConfig::instance()
+KdeConnectConfig& KdeConnectConfig::instance()
 {
-    static KdeConnectConfig* kcc = new KdeConnectConfig();
+    static KdeConnectConfig kcc;
     return kcc;
 }
 
@@ -68,9 +75,8 @@ KdeConnectConfig::KdeConnectConfig()
         Daemon::instance()->reportError(
                              i18n("KDE Connect failed to start"),
                              i18n("Could not find support for RSA in your QCA installation. If your "
-                                  "distribution provides separate packages for QCA-ossl and QCA-gnupg, "
+                                  "distribution provides separate packets for QCA-ossl and QCA-gnupg, "
                                   "make sure you have them installed and try again."));
-        return;
     }
 
     //Make sure base directory exists
@@ -80,77 +86,16 @@ KdeConnectConfig::KdeConnectConfig()
     d->m_config = new QSettings(baseConfigDir().absoluteFilePath(QStringLiteral("config")), QSettings::IniFormat);
     d->m_trustedDevices = new QSettings(baseConfigDir().absoluteFilePath(QStringLiteral("trusted_devices")), QSettings::IniFormat);
 
-    const QFile::Permissions strict = QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::WriteUser;
+    d->m_defaultName = Daemon::instance()->defaultName();
+    d->m_deviceType = Daemon::instance()->deviceType();
 
-    QString keyPath = privateKeyPath();
-    QFile privKey(keyPath);
-    if (privKey.exists() && privKey.open(QIODevice::ReadOnly)) {
-
-        d->m_privateKey = QCA::PrivateKey::fromPEM(privKey.readAll());
-
-    } else {
-
-        d->m_privateKey = QCA::KeyGenerator().createRSA(2048);
-
-        if (!privKey.open(QIODevice::ReadWrite | QIODevice::Truncate))  {
-            Daemon::instance()->reportError(QStringLiteral("KDE Connect"), i18n("Could not store private key file: %1", keyPath));
-        } else {
-            privKey.setPermissions(strict);
-            privKey.write(d->m_privateKey.toPEM().toLatin1());
-        }
-    }
-
-    QString certPath = certificatePath();
-    QFile cert(certPath);
-    if (cert.exists() && cert.open(QIODevice::ReadOnly)) {
-
-        d->m_certificate = QSslCertificate::fromPath(certPath).at(0);
-
-    } else {
-
-        // No certificate yet. Probably first run. Let's generate one!
-
-        QString uuid = QUuid::createUuid().toString();
-        DbusHelper::filterNonExportableCharacters(uuid);
-        qCDebug(KDECONNECT_CORE) << "My id:" << uuid;
-
-        // FIXME: We only use QCA here to generate the cert and key, would be nice to get rid of it completely.
-        // The same thing we are doing with QCA could be done invoking openssl (altought it's potentially less portable):
-        // openssl req -new -x509 -sha256 -newkey rsa:2048 -nodes -keyout privateKey.pem -days 3650 -out certificate.pem -subj "/O=KDE/OU=KDE Connect/CN=_e6e29ad4_2b31_4b6d_8f7a_9872dbaa9095_"
-
-        QCA::CertificateOptions certificateOptions = QCA::CertificateOptions();
-        QDateTime startTime = QDateTime::currentDateTime().addYears(-1);
-        QDateTime endTime = startTime.addYears(10);
-        QCA::CertificateInfo certificateInfo;
-        certificateInfo.insert(QCA::CommonName, uuid);
-        certificateInfo.insert(QCA::Organization,QStringLiteral("KDE"));
-        certificateInfo.insert(QCA::OrganizationalUnit,QStringLiteral("Kde connect"));
-        certificateOptions.setInfo(certificateInfo);
-        certificateOptions.setFormat(QCA::PKCS10);
-        certificateOptions.setSerialNumber(QCA::BigInteger(10));
-        certificateOptions.setValidityPeriod(startTime, endTime);
-
-        d->m_certificate = QSslCertificate(QCA::Certificate(certificateOptions, d->m_privateKey).toPEM().toLatin1());
-
-        if (!cert.open(QIODevice::ReadWrite | QIODevice::Truncate))  {
-            Daemon::instance()->reportError(QStringLiteral("KDE Connect"), i18n("Could not store certificate file: %1", certPath));
-        } else {
-            cert.setPermissions(strict);
-            cert.write(d->m_certificate.toPem());
-        }
-    }
-
-    //Extra security check
-    if (QFile::permissions(keyPath) != strict) {
-        qCWarning(KDECONNECT_CORE) << "Warning: KDE Connect private key file has too open permissions " << keyPath;
-    }
+    loadPrivateKey();
+    loadCertificate();
 }
 
 QString KdeConnectConfig::name()
 {
-    QString defaultName = qgetenv("USER") + '@' + QHostInfo::localHostName();
-    QString name = d->m_config->value(QStringLiteral("name"), defaultName).toString();
-    return name;
+    return d->m_config->value(QStringLiteral("name"), d->m_defaultName).toString();
 }
 
 void KdeConnectConfig::setName(const QString& name)
@@ -161,12 +106,12 @@ void KdeConnectConfig::setName(const QString& name)
 
 QString KdeConnectConfig::deviceType()
 {
-    return QStringLiteral("desktop"); // TODO
+    return d->m_deviceType;
 }
 
 QString KdeConnectConfig::deviceId()
 {
-    return d->m_certificate.subjectInfo( QSslCertificate::CommonName ).constFirst();
+    return d->m_certificate.subjectInfo(QSslCertificate::CommonName).constFirst();
 }
 
 QString KdeConnectConfig::privateKeyPath()
@@ -174,16 +119,6 @@ QString KdeConnectConfig::privateKeyPath()
     return baseConfigDir().absoluteFilePath(QStringLiteral("privateKey.pem"));
 }
 
-QCA::PrivateKey KdeConnectConfig::privateKey()
-{
-    return d->m_privateKey;
-}
-
-QCA::PublicKey KdeConnectConfig::publicKey()
-{
-    return d->m_privateKey.toPublicKey();
-}
-
 QString KdeConnectConfig::certificatePath()
 {
     return baseConfigDir().absoluteFilePath(QStringLiteral("certificate.pem"));
@@ -196,7 +131,7 @@ QSslCertificate KdeConnectConfig::certificate()
 
 QDir KdeConnectConfig::baseConfigDir()
 {
-    QString configPath = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
+    QString configPath = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation);
     QString kdeconnectConfigPath = QDir(configPath).absoluteFilePath(QStringLiteral("kdeconnect"));
     return QDir(kdeconnectConfigPath);
 }
@@ -270,4 +205,170 @@ QDir KdeConnectConfig::pluginConfigDir(const QString& deviceId, const QString& p
     return QDir(pluginConfigDir);
 }
 
+void KdeConnectConfig::loadPrivateKey()
+{
+    QString keyPath = privateKeyPath();
+    QFile privKey(keyPath);
 
+    bool needsToGenerateKey = false;
+    if (privKey.exists() && privKey.open(QIODevice::ReadOnly)) {
+        QCA::ConvertResult result;
+        d->m_privateKey = QCA::PrivateKey::fromPEM(QString::fromLatin1(privKey.readAll()), QCA::SecureArray(), &result);
+        if (result != QCA::ConvertResult::ConvertGood) {
+            qCWarning(KDECONNECT_CORE) << "Private key from" << keyPath << "is not valid";
+            needsToGenerateKey = true;
+        }
+    } else {
+        needsToGenerateKey = true;
+    }
+
+    if (needsToGenerateKey) {
+        generatePrivateKey(keyPath);
+    }
+
+    //Extra security check
+    if (QFile::permissions(keyPath) != strictPermissions) {
+        qCWarning(KDECONNECT_CORE) << "Warning: KDE Connect private key file has too open permissions " << keyPath;
+    }
+}
+
+void KdeConnectConfig::loadCertificate()
+{
+    QString certPath = certificatePath();
+    QFile cert(certPath);
+
+    bool needsToGenerateCert = false;
+    if (cert.exists() && cert.open(QIODevice::ReadOnly)) {
+        auto loadedCerts = QSslCertificate::fromPath(certPath);
+        if (loadedCerts.empty()) {
+            qCWarning(KDECONNECT_CORE) << "Certificate from" << certPath << "is not valid";
+            needsToGenerateCert = true;
+        } else {
+            d->m_certificate = loadedCerts.at(0);
+        }
+    } else {
+        needsToGenerateCert = true;
+    }
+
+    if (needsToGenerateCert) {
+        generateCertificate(certPath);
+    }
+
+    //Extra security check
+    if (QFile::permissions(certPath) != strictPermissions) {
+        qCWarning(KDECONNECT_CORE) << "Warning: KDE Connect certificate file has too open permissions " << certPath;
+    }
+}
+
+void KdeConnectConfig::generatePrivateKey(const QString& keyPath)
+{
+    qCDebug(KDECONNECT_CORE) << "Generating private key";
+
+    bool error = false;
+
+    d->m_privateKey = QCA::KeyGenerator().createRSA(2048);
+
+    QFile privKey(keyPath);
+    if (!privKey.open(QIODevice::ReadWrite | QIODevice::Truncate))  {
+        error = true;
+    } else {
+        privKey.setPermissions(strictPermissions);
+        int written = privKey.write(d->m_privateKey.toPEM().toLatin1());
+        if (written <= 0) {
+            error = true;
+        }
+    }
+
+    if (error) {
+        Daemon::instance()->reportError(QStringLiteral("KDE Connect"), i18n("Could not store private key file: %1", keyPath));
+    }
+
+}
+
+void KdeConnectConfig::generateCertificate(const QString& certPath)
+{
+    qCDebug(KDECONNECT_CORE) << "Generating certificate";
+
+    bool error = false;
+
+    QString uuid = QUuid::createUuid().toString();
+    DBusHelper::filterNonExportableCharacters(uuid);
+    qCDebug(KDECONNECT_CORE) << "My id:" << uuid;
+
+    // FIXME: We only use QCA here to generate the cert and key, would be nice to get rid of it completely.
+    // The same thing we are doing with QCA could be done invoking openssl (although it's potentially less portable):
+    // openssl req -new -x509 -sha256 -newkey rsa:2048 -nodes -keyout privateKey.pem -days 3650 -out certificate.pem -subj "/O=KDE/OU=KDE Connect/CN=_e6e29ad4_2b31_4b6d_8f7a_9872dbaa9095_"
+
+    QCA::CertificateOptions certificateOptions = QCA::CertificateOptions();
+    QDateTime startTime = QDateTime::currentDateTime().addYears(-1);
+    QDateTime endTime = startTime.addYears(10);
+    QCA::CertificateInfo certificateInfo;
+    certificateInfo.insert(QCA::CommonName, uuid);
+    certificateInfo.insert(QCA::Organization,QStringLiteral("KDE"));
+    certificateInfo.insert(QCA::OrganizationalUnit,QStringLiteral("Kde connect"));
+    certificateOptions.setInfo(certificateInfo);
+    certificateOptions.setFormat(QCA::PKCS10);
+    certificateOptions.setSerialNumber(QCA::BigInteger(10));
+    certificateOptions.setValidityPeriod(startTime, endTime);
+
+    d->m_certificate = QSslCertificate(QCA::Certificate(certificateOptions, d->m_privateKey).toPEM().toLatin1());
+
+    QFile cert(certPath);
+    if (!cert.open(QIODevice::ReadWrite | QIODevice::Truncate))  {
+        error = true;
+    } else {
+        cert.setPermissions(strictPermissions);
+        int written = cert.write(d->m_certificate.toPem());
+        if (written <= 0) {
+            error = true;
+        }
+    }
+
+    if (error) {
+        Daemon::instance()->reportError(QStringLiteral("KDE Connect"), i18n("Could not store certificate file: %1", certPath));
+    }
+}
+
+#ifdef USE_PRIVATE_DBUS
+QString KdeConnectConfig::privateDBusAddressPath()
+{
+    return baseConfigDir().absoluteFilePath(QStringLiteral("private_dbus_address"));
+}
+
+QString KdeConnectConfig::privateDBusAddress()
+{
+    if (d->m_privateDBusAddress.length() != 0) return d->m_privateDBusAddress;
+
+    QString dbusAddressPath = privateDBusAddressPath();
+    QFile dbusAddressFile(dbusAddressPath);
+
+    if (!dbusAddressFile.open(QFile::ReadOnly | QFile::Text)) {
+        qCCritical(KDECONNECT_CORE) << "Private DBus enabled but error read private dbus address conf";
+        exit(1);
+    }
+
+    QTextStream in(&dbusAddressFile);
+
+    qCDebug(KDECONNECT_CORE) << "Waiting for private dbus";
+
+    int retry = 0;
+    QString addr = in.readLine();
+    while(addr.length() == 0 && retry < 5) {
+        qCDebug(KDECONNECT_CORE) << "Retry reading private DBus address after 3s";
+        QThread::sleep(3);
+        retry ++;
+        addr = in.readLine();   // Read until first not empty line
+    }
+
+    if (addr.length() == 0) {
+        qCCritical(KDECONNECT_CORE) << "Private DBus enabled but read private dbus address failed";
+        exit(1);
+    }
+
+    qCDebug(KDECONNECT_CORE) << "Private dbus address: " << addr;
+
+    d->m_privateDBusAddress = addr;
+
+    return addr;
+}
+#endif
