@@ -28,6 +28,8 @@
 #include <QSessionManager>
 #include <QStandardPaths>
 #include <QIcon>
+#include <QProcess>
+#include <QHostInfo>
 
 #include <KAboutData>
 #include <KDBusService>
@@ -65,9 +67,13 @@ public:
         notification->setComponentName(QStringLiteral("kdeconnect"));
         notification->setTitle(QStringLiteral("KDE Connect"));
         notification->setText(i18n("Pairing request from %1", device->name().toHtmlEscaped()));
+        notification->setDefaultAction(i18n("Open"));
         notification->setActions(QStringList() << i18n("Accept") << i18n("Reject"));
         connect(notification, &KNotification::action1Activated, device, &Device::acceptPairing);
         connect(notification, &KNotification::action2Activated, device, &Device::rejectPairing);
+        connect(notification, QOverload<>::of(&KNotification::activated), this, []{
+            QProcess::startDetached(QStringLiteral("kdeconnect-settings"));
+        });
         notification->sendEvent();
     }
 
@@ -90,6 +96,20 @@ public:
         return KIO::getJobTracker();
     }
 
+    QString defaultName() const override {
+        QString username;
+#ifdef Q_OS_WIN
+        username = QString::fromLatin1(qgetenv("USERNAME"));
+#else
+        username = QString::fromLatin1(qgetenv("USER"));
+#endif
+        return username + QLatin1Char('@') + QHostInfo::localHostName();
+    }
+
+    QString deviceType() const override {
+        return QStringLiteral("desktop");
+    }
+
     Q_SCRIPTABLE void sendSimpleNotification(const QString &eventId, const QString &title, const QString &text, const QString &iconName) override
     {
         KNotification* notification = new KNotification(eventId); //KNotification::Persistent
