@@ -15,28 +15,28 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "pausemusicplugin.h"
 
-#include <QDBusConnection>
-#include <QDBusInterface>
-#include <QDBusConnectionInterface>
-#include <QDBusMessage>
-#include <QDBusReply>
-#include <QDebug>
-#include <QProcess>
-
 #include <KPluginFactory>
+#include <PulseAudioQt/Context>
+#include <PulseAudioQt/Sink>
 
-K_PLUGIN_FACTORY_WITH_JSON( KdeConnectPluginFactory, "kdeconnect_pausemusic.json", registerPlugin< PauseMusicPlugin >(); )
+#include <dbushelper.h>
+#include "mprisplayer.h"
+
+//In older Qt released, qAsConst isnt available
+#include "qtcompat_p.h"
+
+K_PLUGIN_CLASS_WITH_JSON(PauseMusicPlugin, "kdeconnect_pausemusic.json")
 
 Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_PAUSEMUSIC, "kdeconnect.plugin.pausemusic")
 
 PauseMusicPlugin::PauseMusicPlugin(QObject* parent, const QVariantList& args)
     : KdeConnectPlugin(parent, args)
-    , muted(false)
+    , mutedSinks()
 {}
 
 bool PauseMusicPlugin::receivePacket(const NetworkPacket& np)
@@ -62,24 +62,29 @@ bool PauseMusicPlugin::receivePacket(const NetworkPacket& np)
 
         if (mute) {
             qCDebug(KDECONNECT_PLUGIN_PAUSEMUSIC) << "Muting system volume";
-            QProcess::startDetached("pactl set-sink-mute @DEFAULT_SINK@ 1");
-            muted = true;
+            const auto sinks = PulseAudioQt::Context::instance()->sinks();
+            for (const auto sink : sinks) {
+                if (!sink->isMuted()) {
+                    sink->setMuted(true);
+                    mutedSinks.insert(sink->name());
+                }
+            }
         }
 
         if (pause) {
             //Search for interfaces currently playing
-            const QStringList interfaces = QDBusConnection::sessionBus().interface()->registeredServiceNames().value();
+            const QStringList interfaces = DBusHelper::sessionBus().interface()->registeredServiceNames().value();
             for (const QString& iface : interfaces) {
                 if (iface.startsWith(QLatin1String("org.mpris.MediaPlayer2"))) {
-                    QDBusInterface mprisInterface(iface, QStringLiteral("/org/mpris/MediaPlayer2"), QStringLiteral("org.mpris.MediaPlayer2.Player"));
-                    QString status = mprisInterface.property("PlaybackStatus").toString();
+                    OrgMprisMediaPlayer2PlayerInterface mprisInterface(iface, QStringLiteral("/org/mpris/MediaPlayer2"), DBusHelper::sessionBus());
+                    QString status = mprisInterface.playbackStatus();
                     if (status == QLatin1String("Playing")) {
                         if (!pausedSources.contains(iface)) {
                             pausedSources.insert(iface);
-                            if (mprisInterface.property("CanPause").toBool()) {
-                                mprisInterface.asyncCall(QStringLiteral("Pause"));
+                            if (mprisInterface.canPause()) {
+                                mprisInterface.Pause();
                             } else {
-                                mprisInterface.asyncCall(QStringLiteral("Stop"));
+                                mprisInterface.Stop();
                             }
                         }
                     }
@@ -89,18 +94,23 @@ bool PauseMusicPlugin::receivePacket(const NetworkPacket& np)
 
     } else {
 
-        if (mute && muted) {
+        if (mute) {
 
             qCDebug(KDECONNECT_PLUGIN_PAUSEMUSIC) << "Unmuting system volume";
-            QProcess::startDetached("pactl set-sink-mute @DEFAULT_SINK@ 0");
 
-            muted = false;
+            const auto sinks = PulseAudioQt::Context::instance()->sinks();
+            for (const auto sink : sinks) {
+                if (mutedSinks.contains(sink->name())) {
+                    sink->setMuted(false);
+                }
+            }
+            mutedSinks.clear();
         }
 
         if (pause && !pausedSources.empty()) {
             for (const QString& iface : qAsConst(pausedSources)) {
-                QDBusInterface mprisInterface(iface, QStringLiteral("/org/mpris/MediaPlayer2"), QStringLiteral("org.mpris.MediaPlayer2.Player"));
-                mprisInterface.asyncCall(QStringLiteral("Play"));
+                OrgMprisMediaPlayer2PlayerInterface mprisInterface(iface, QStringLiteral("/org/mpris/MediaPlayer2"), DBusHelper::sessionBus());
+                mprisInterface.Play();
             }
             pausedSources.clear();
         }
